#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"

#define LINEPIX 1024
#define MAXCHAR 1024
#define MAXHDR 1024
#define HDRSIZE 80
#define ITERATION 3
#define NSIGMA 3

/* pixel coordinate is saved in array as */
/* pix[y][x] */
/* naxes[0]=xmax */
/* naxes[1]=ymax */

main(int argc, char **argv)
{
  FILE *fp;
  fitsfile *fptr1,*fptr2;
  int status, nfound, anynull;
  long fpixel, npixels;
  int i,j,k,l,h,n;
  float nullval; 
  char openfile[100],savefile[100],medifile[100];

  int bitpix=-32;
  long naxis=2;

  long naxes[2]={LINEPIX, LINEPIX};
  long naxes2[2]={LINEPIX, LINEPIX};
  static float buffer[LINEPIX],pix[LINEPIX][LINEPIX],spix[LINEPIX][LINEPIX];
  long nbuffer=LINEPIX;

  char line[MAXCHAR];
 
  int xwid,ywid,dx,dy;
  double sum,sum2,av,sigma;
  double nsigp,nsign;
  int num;
  double lpix;

  int nkeys;
  char *hdrptr[MAXHDR];

  if(argc!=6 && argc!=7){
    fprintf(stderr,"Usage : %s (input file) (output file) (xwid) (nsigma+) (nsigma-) [(ywid)]\n",argv[0]);
    fprintf(stderr,"      : xwid is the width of the box to calculate sigma value for each column.\n");
    fprintf(stderr,"      : ywid is the height of the same box : if omitted, sigma of the whole column will be used instead.\n");
    fprintf(stderr,"      : nsigma+/- is the thershould to define bad pixel for positive/negative pixel.\n");
    exit(1);
  }

  sprintf(openfile, "%s",argv[1]);
  sprintf(savefile, "%s",argv[2]);
  xwid=atoi(argv[3]);
  nsigp=atof(argv[4]);
  nsign=atof(argv[5]);
  if(argc==7) ywid=atoi(argv[6]);
  else ywid=0;

  status=0;

  if(fits_open_file(&fptr1,openfile, READONLY, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /* Read Header Information */
  if(fits_read_keys_lng(fptr1, "NAXIS", 1, 2, naxes, &nfound, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if((naxes[0]>LINEPIX)||(naxes[1]>LINEPIX)){
    fprintf(stderr,"Image is too large\n");
    exit(1);
  }
  
  if(fits_get_hdrspace(fptr1, &nkeys, NULL, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    hdrptr[i]=malloc(HDRSIZE);
    if(fits_read_record(fptr1,i+1,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  /* Read Input Frame */
  fprintf(stderr,"naxes[0]=%d naxes[1]=%d\n",naxes[0],naxes[1]);

  nbuffer=naxes[0];
  fpixel = 1;
  nullval = 0;
  for(j=0;j<naxes[1];j++){
    if (fits_read_img(fptr1, TFLOAT, fpixel, nbuffer, &nullval,
		      buffer, &anynull, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    for(i=0;i<naxes[0];i++) pix[j][i]=buffer[i];
    fpixel+=nbuffer;
  }
    
  if(fits_close_file(fptr1, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  fprintf(stderr,"pix[0][0]=%f pix[0][1]=%f\n",pix[0][0],pix[0][1]);
  fprintf(stderr,"pix[1][0]=%f pix[1][1]=%f\n",pix[1][0],pix[1][1]);

  fprintf(stderr,"OK1\n");

  /* Calculate noise for each column */
  dx=(xwid-1)/2;
  dy=(ywid-1)/2;
  
  fprintf(stderr,"dx=%d dy=%d\n",dx,dy);
  for(i=0;i<naxes[0];i++){ 
    sigma=1e30;
    av=0;
    if(ywid==0){
      /* calculate sigma of whole row */
      /* Start iteration to calculate sigma */
      for(h=0;h<ITERATION;h++){
	sum=0.0;
	sum2=0.0;
	num=0;
	for(k=-dx;k<=dx;k++){
	  if(i+k>=0 && i+k<1024)
	    for(j=0;j<naxes[1];j++){
	      lpix=pix[j][i+k];
	      if(lpix<av+NSIGMA*sigma && lpix>av-NSIGMA*sigma){
		sum+=lpix;
		sum2+=lpix*lpix;
		num++;
	      }
	    }
	}
	av=sum/num;
	sigma=sqrt(sum2/num-av*av);
      }
      for(j=0;j<naxes[1];j++){
	if(pix[j][i]>=nsigp*sigma || pix[j][i]<=-nsign*sigma) spix[j][i]=1.0;
	else spix[j][i]=0.0;
      }
    }
    else{
      /* calculate sigma of the (xwid x ywid) box */
      /* Start iteration to calculate sigma */
      for(j=0;j<naxes[1];j++){
	for(h=0;h<ITERATION;h++){
	  sum=0.0;
	  sum2=0.0;
	  num=0;
	  for(k=i-dx;k<=i+dx;k++){
	    if(k>=0 && k<naxes[0]){
	      for(l=j-dy;l<=j+dy;l++){
		if(l>=0 && l<naxes[1]){
		  if(pix[l][k]<av+NSIGMA*sigma && pix[l][k]>av-NSIGMA*sigma){
		    sum+=pix[l][k];
		    sum2+=pix[l][k]*pix[l][k];
		    num++;
		  }
		}
	      }
	    }
	  }
	  av=sum/num;
	  sigma=sqrt(sum2/num-av*av);
	}
	if(pix[j][i]>=nsigp*sigma || pix[j][i]<=-nsign*sigma) spix[j][i]=1.0;
	else spix[j][i]=0.0;
      }
    }
  }

  printf("\t: Write %s  ... ",savefile);
  unlink(savefile);

  status=0;

  if (fits_create_file(&fptr2, savefile, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  for (i=0;i<nkeys;i++){
    if(fits_write_record(fptr2,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  fpixel=1;

  npixels=naxes[0];
  fprintf(stderr,"%d\n",npixels);
  for(j=0;j<naxes[1];j++){
    if (fits_write_img(fptr2, TFLOAT, fpixel, npixels, spix[j], &status)) 
	  fits_report_error(stderr,status);
    fpixel+=naxes[0];
  }

  fprintf(stderr,"%d\n",npixels);

  if (fits_close_file(fptr2, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");

}
