#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"

#define LINEPIX 1024
#define MAXCHAR 1024
#define MAXHDR 1024
#define HDRSIZE 80

main(int argc, char **argv)
{
  FILE *fp;
  fitsfile *fptr1,*fptr2;
  int status, nfound, anynull;
  long fpixel, npixels;
  int i,j;
  float nullval; 
  char openfile[100],savefile[100],paramfile[100];
  char grism[100];
  char filter02[64];

  int bitpix=-32;
  long naxis=2;

  long naxes[2]={LINEPIX, LINEPIX};
  static float buffer[LINEPIX],pix[LINEPIX][LINEPIX];
  long nbuffer=LINEPIX;

  float a,b,x0,xoffset;
  char line[MAXCHAR];
  char disperser[MAXCHAR];
 
  float y,subpixshift,residual;
  int pixshift;

  int nkeys;
  char *hdrptr[MAXHDR];

  if(argc!=4 && argc !=3){
    fprintf(stderr,"Usage : %s (input file) (output file) [zJGr/JHGr/wKGr/wKGr_old]\n",argv[0]);
    exit(1);
  }

  sprintf(openfile, "%s",argv[1]);
  sprintf(savefile, "%s",argv[2]);
  if(argc==4) sprintf(grism, "%s",argv[3]);
  
  printf("%s \t: Read %s  ... ",argv[0],openfile);

  status=0;

  if(fits_open_file(&fptr1,openfile, READONLY, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(argc==3){
    if(fits_read_key_str(fptr1, "FILTER02" , grism, NULL, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    fprintf(stderr,"### According to the header, grism is %s ###\n",grism);
  }
  if(strcmp(grism,"zJGr")==0){a=-2.678954e-05; b=3.069337e-02;}
  else if(strcmp(grism,"JHGr")==0){a=-2.197630e-05; b=2.460170e-02;}
/*  else if(strcmp(grism,"wKGr")==0){a=-3.03244e-05; b=0.0358957;} */
  else if(strcmp(grism,"wKGr")==0){a=-3.135410e-05; b=3.605263e-02;}
  else if(strcmp(grism,"wKGr_old")==0){a=-3.42834e-05; b=0.0445231;}
  else{
    fprintf(stderr,"Error : Grism should be zJGr, JHGr, wKGr or wKGr_old.\n");
    exit(1);
  }
  x0=-b/2.0/a;
  fprintf(stderr,"x0=%f\n",x0);
  x0=513;

  /* Read Header Information */
  if(fits_read_keys_lng(fptr1, "NAXIS", 1, 2, naxes, &nfound, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if((naxes[0]>LINEPIX)||(naxes[1]>LINEPIX)){
    fprintf(stderr,"Image is too large\n");
    exit(0);
  }
  
  if(fits_get_hdrspace(fptr1, &nkeys, NULL, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    hdrptr[i]=malloc(HDRSIZE);
    if(fits_read_record(fptr1,i+1,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }


  nbuffer=naxes[1];
  fpixel = 1;
  nullval = 0;
  for(i=0;i<naxes[0];i++){
    if (fits_read_img(fptr1, TFLOAT, fpixel, nbuffer, &nullval,
		      buffer, &anynull, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    y=(float)i+1.0;
    pixshift=(int) floor(a*(y*y-x0*x0)+b*(y-x0));
    subpixshift=a*(y*y-x0*x0)+b*(y-x0)-(float) pixshift;
    for(j=0;j<naxes[1];j++){
      if(((j+pixshift)<0)||((j+pixshift)>1022)) pix[i][j]=0;
      else pix[i][j]=buffer[j+pixshift+1]*subpixshift+buffer[j+pixshift]*(1.0-subpixshift);
    }
    fpixel+=nbuffer;
  }
    
  if(fits_close_file(fptr1, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");


  printf("\t: Write %s  ... ",savefile);
  unlink(savefile);

  status=0;

  if (fits_create_file(&fptr2, savefile, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /*  if (fits_create_img(fptr2, bitpix, naxis, naxes, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  */
  for (i=0;i<nkeys;i++){
    if(fits_write_record(fptr2,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  fpixel=1;

  npixels=naxes[1];
  for(i=0;i<naxes[0];i++){
    if (fits_write_img(fptr2, TFLOAT, fpixel, npixels, pix[i], &status)) 
	  fits_report_error(stderr,status);
    fpixel+=naxes[1];
  }

  if (fits_close_file(fptr2, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");

}
